/*
 * Copyright (C) 2023 iopsys Software Solutions AB
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2.1
 * as published by the Free Software Foundation
 *
 *	  Author: Suvendhu Hansa <suvendhu.hansa@iopsys.eu>
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libubus.h>
#include <libubox/uloop.h>

#include "helper.h"

struct ubus_context *g_ubus_ctx;

static int timemngr_info(struct ubus_context *ctx, struct ubus_object *obj,
			struct ubus_request_data *req, const char *method,
			struct blob_attr *msg);

enum {
	TIMEMNGR_OBJECT,
	TIMEMNGR_SECTION,
	__TIMEMNGR_PARAM_MAX
};

static const struct blobmsg_policy timemngr_policy[] = {
	[TIMEMNGR_OBJECT] = { .name = "object", .type = BLOBMSG_TYPE_STRING },
	[TIMEMNGR_SECTION] = { .name = "section", .type = BLOBMSG_TYPE_STRING }
};

static const struct ubus_method timemngr_object_methods[__TIMEMNGR_PARAM_MAX] = {
	UBUS_METHOD("status", timemngr_info, timemngr_policy),
	UBUS_METHOD("stats", timemngr_info, timemngr_policy)
};

static struct ubus_object_type timemngr_object_type = UBUS_OBJECT_TYPE("timemngr", timemngr_object_methods);

static struct ubus_object timemngr_object = {
	.name = "timemngr",
	.type = &timemngr_object_type,
	.methods = timemngr_object_methods,
	.n_methods = ARRAY_SIZE(timemngr_object_methods)
};

static int timemngr_info(struct ubus_context *ctx, struct ubus_object *obj,
			struct ubus_request_data *req, const char *method,
			struct blob_attr *msg)
{
	struct blob_attr *tb[__TIMEMNGR_PARAM_MAX] = {NULL};

	if (blobmsg_parse(timemngr_policy, __TIMEMNGR_PARAM_MAX, tb, blob_data(msg), blob_len(msg)))
		return UBUS_STATUS_UNKNOWN_ERROR;

	if (!tb[TIMEMNGR_OBJECT])
		return UBUS_STATUS_INVALID_ARGUMENT;

	char *object = blobmsg_get_string(tb[TIMEMNGR_OBJECT]);
	char *section = tb[TIMEMNGR_SECTION] ? blobmsg_get_string(tb[TIMEMNGR_SECTION]) : NULL;

	PRINT_DEBUG("Ubus call for method: (%s), object: (%s), section: (%s)", method, object, section);
	struct blob_buf bb;
	memset(&bb, 0, sizeof(struct blob_buf));
	blob_buf_init(&bb, 0);

	if (strcmp(method, "status") == 0) {
		get_current_status(object, section, &bb);
	} else if (strcmp(method, "stats") == 0) {
		get_current_stats(object, section, &bb);
	} else {
		blob_buf_free(&bb);
		return UBUS_STATUS_UNKNOWN_ERROR;
	}
	PRINT_DEBUG("Ubus call complete, result: %s", blobmsg_format_json(bb.head, true));

	ubus_send_reply(ctx, req, bb.head);
	blob_buf_free(&bb);

	return 0;
}


static int register_methods(struct ubus_context *ubus_ctx)
{
	int ret = ubus_add_object(ubus_ctx, &timemngr_object);
	if (ret)
		PRINT_ERR("Failed to register ubus methods");

	return ret;
}

static void timemngr_usage(char *n)
{
	fprintf(stdout,
		"\nUsage: %s: <options>\n"
		"Options:\n"
		"  -l <loglevel> log verbosity value as per standard syslog\n"
		"  -h help\n\n"
		, n);
}

int main(int argc, char **argv)
{
	int c = -1;
	int log_level = 3; // Default is LOG_ERR

	while ((c = getopt(argc, argv, "l:h")) != -1) {
		switch (c) {
		case 'h':
			timemngr_usage(argv[0]);
			return EXIT_SUCCESS;
		case 'l':
			log_level = (int)strtod(optarg, NULL);
			if (log_level < 0 || log_level > 7)
				log_level = 3;
			break;
		default:
			timemngr_usage(argv[0]);
			return EXIT_FAILURE;
		}
	}

	setlogmask(LOG_UPTO(log_level));
	openlog("timemngr", LOG_CONS | LOG_PID | LOG_NDELAY, LOG_LOCAL1);

	uloop_init();
	g_ubus_ctx = ubus_connect(NULL);
	if (!g_ubus_ctx) {
		PRINT_ERR("Failed to connect to ubus");
		return -1;
	}

	ubus_add_uloop(g_ubus_ctx);
	if (register_methods(g_ubus_ctx))
		goto out;

	start_sync_timer(10);
	uloop_run();

out:
	ubus_free(g_ubus_ctx);
	uloop_done();

	return 0;
}
